#include "Agent.hpp"

#include <stdlib.h>

Agent::Agent(float fl_x, float fl_y, float fl_w, float fl_h, float fl_direction, float fl_pheromone_strength, float fl_random_influence, float fl_lookahead_distance)
{
    this->fl_x = fl_x;
    this->fl_y = fl_y;
    this->fl_w = fl_w;
    this->fl_h = fl_h;
    this->fl_direction = fl_direction;
    this->fl_pheromone_strength = fl_pheromone_strength;
    this->fl_random_influence = 100.0f*fl_random_influence;
    this->fl_lookahead_distance = fl_lookahead_distance;
}

Agent::~Agent()
{

}

void Agent::tick(Field* field)
{
    ///Update direction
    float fl_xd, fl_yd;

    if ((float)(rand()%100) < fl_random_influence)
    {   //Move randomly
        switch (rand() % 3)
        {
            case 0:
                fl_direction += M_PI_4;
                break;
            case 1:
                break;
            case 2:
                fl_direction -= M_PI_4;
                break;
        }
    }
    else
    {   //Move based on environment

        //Gather information
        float fl_l_pheromone, fl_s_pheromone, fl_r_pheromone, fl_test_dir = fl_direction;
        fl_xd = fl_lookahead_distance*cos(fl_test_dir);
        fl_yd = fl_lookahead_distance*sin(fl_test_dir);

        fl_s_pheromone = field->get_pheromone_strength(fl_x+fl_xd, fl_y+fl_yd);

        fl_test_dir += M_PI/6.0F;
        fl_xd = fl_lookahead_distance*cos(fl_test_dir);
        fl_yd = fl_lookahead_distance*sin(fl_test_dir);
        fl_l_pheromone = field->get_pheromone_strength(fl_x+fl_xd, fl_y+fl_yd);

        fl_test_dir -= M_PI/3.0F;
        fl_xd = fl_lookahead_distance*cos(fl_test_dir);
        fl_yd = fl_lookahead_distance*sin(fl_test_dir);
        fl_r_pheromone = field->get_pheromone_strength(fl_x+fl_xd, fl_y+fl_yd);

        if (fl_r_pheromone >= fl_s_pheromone && fl_r_pheromone >= fl_l_pheromone)
        {
            fl_direction -= M_PI/6.0;
        }
        else if (fl_l_pheromone >= fl_s_pheromone && fl_l_pheromone >= fl_r_pheromone)
        {
            fl_direction += M_PI/6.0;
        }
    }

    //Move
    fl_xd = cos(fl_direction);
    fl_yd = sin(fl_direction);

    if (fl_x + fl_xd < 0 || fl_x + fl_xd >= fl_w || fl_y + fl_yd < 0 || fl_y + fl_yd >= fl_h)
    {
        fl_x = fmin(fl_w,fmax(0.0f, fl_x+fl_xd));
        fl_y = fmin(fl_h,fmax(0.0f, fl_y+fl_yd));
        fl_direction = 2.0f*M_PI*((float)(rand()%100)/100.0f);
    }
    else
    {
        fl_x += fl_xd;
        fl_y += fl_yd;
    }

    //Add pheromone to new position
    field->add_pheromone(fl_x, fl_y, fl_pheromone_strength);
}