#include "Field.hpp"

#include <string.h>

Field::Field(int i_width, int i_height, float fl_pheromone_max)
{
    this->i_width = i_width;
    this->i_height = i_height;

    fl_ground = new float*[i_width];

    for (int i = 0; i < i_width; i++)
    {
        fl_ground[i] = new float[i_height];

        for(int j = 0; j < i_height; j++)
        {
            fl_ground[i][j] = 0.0f;
        }
    }

    this->fl_pheromone_max = fl_pheromone_max;
}

Field::~Field()
{
    for(int i = 0; i < i_width; i++)
    {
        delete[] fl_ground[i];
    }

    delete[] fl_ground;
}

void Field::tick()
{
    float** fl_temp_ground;
    fl_temp_ground = new float*[i_width];

    for (int i = 0; i < i_width; i++)
    {
        fl_temp_ground[i] = new float[i_height];
    }


    for(int x = 0; x < i_width; x++)
    {
        for(int y = 0; y < i_height; y++)
        {

            float fl_sum = 0.0f;
            float fl_count = 0.0f;

            for (int i = x-1; i <= x+1; i++)
            {
                for (int j = y-1; j <= y+1; j++)
                {
                    if (i < 0 || i >= i_width || j < 0 || j >= i_height)
                    {
                        continue;
                    }

                    fl_sum += fl_ground[i][j];
                    fl_count += 1.0f;
                }
            }

            fl_temp_ground[x][y] = fmax(0.0f, fl_sum/fl_count-0.005);
        }
    }

    for(int i = 0; i < i_width; i++)
    {
        memcpy(&fl_ground[i][0], &fl_temp_ground[i][0], sizeof(float)*i_height);
        delete[] fl_temp_ground[i];
    }

    delete[] fl_temp_ground;
}

void Field::add_pheromone(float fl_x, float fl_y, float fl_strength)
{
    int i_x = (int)fl_x;
    int i_y = (int)fl_y;

    if (i_x < 0 || i_x >= i_width || i_y < 0 || i_y >= i_height)
    {
        return;
    }

    fl_ground[i_x][i_y] = fmin(fl_ground[i_x][i_y]+fl_strength, fl_pheromone_max);
}

float Field::get_pheromone_strength(float fl_x, float fl_y)
{
    int i_x = (int)fl_x, i_y = (int)fl_y;
    if (i_x < 0 || i_x >= i_width || i_y < 0 || i_y >= i_height)
    {
        return 0.0f;
    }

    return fl_ground[i_x][i_y];
}

float Field::get_pheromone_strength(int i_x, int i_y)
{
    if (i_x < 0 || i_x >= i_width || i_y < 0 || i_y >= i_height)
    {
        return 0.0f;
    }

    return fl_ground[i_x][i_y];
}

float Field::get_pheromone_max()
{
    return fl_pheromone_max;
}